# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AdvancedPhoneInformation,
        AdvancedPhoneInformationCarrier,
        AdvancedPhoneInformationContact,
        AdvancedPhoneInformationLocation,
        AdvancedPhoneInformationLocationCoordinates,
        AdvancedPhoneInformationLocationCountry,
        AdvancedPhoneInformationType,
        AdvancedPhoneInformationTypeRecommendation,
        Agent,
        AttachWebhookByIdParams,
        AttachWebhookParams,
        AttachWebhookResponseWebhook,
        AttachedPhoneNumberResult,
        AudienceCountOnly,
        AudienceWithPagination,
        AutofillCampaignParams,
        AutofillDlcCampaignResponse,
        AutofillDlcResponseKeywords,
        AutofillDlcResponseKeywordsHelp,
        AutofillDlcResponseKeywordsOptIn,
        AutofillDlcResponseKeywordsOptOut,
        AutofillDlcResponseLinks,
        AutofillDlcResponseOptions,
        AutofillDlcResponseUseCase,
        BadRequestErrorBody,
        BaseRichMessage,
        BasicPhoneInformation,
        BasicPhoneInformationContact,
        BasicPhoneInformationLocation,
        BasicPhoneInformationLocationCountry,
        BlastDetails,
        BrandStatus,
        BrandStatusEnum,
        ButtonClicked,
        ButtonClickedData,
        ButtonClickedDataButton,
        ButtonClickedDataButtonRaw,
        ButtonClickedDataButtonType,
        BuyResponseCapabilities,
        CampaignEnum,
        CampaignQuery,
        CampaignSubmissionResult,
        CampaignValidationResponseErrorsItem,
        CampaignValidationResult,
        CompanyEntityTypeEnum,
        CompanySectorEnum,
        CompanyTypeEnum,
        ConfiguredWebhook,
        Contact,
        ContactId,
        Conversation,
        ConversationByIdParams,
        ConversationByParticipantsParams,
        ConversationContact,
        ConversationList,
        ConversationSender,
        CreateAndAttachWebhookByUrlParams,
        CreateUrlOptions,
        DeleteAudienceResponse,
        DetachedPhoneNumberResult,
        DetachedWebhookInfo,
        DetailedPhoneNumberEnum,
        DlcAssignmentStatusEnum,
        DlcCampaignStatus,
        DlcCampaignUseCaseEnum,
        DlcCampaignWithExtendedBrandAndStatus,
        DlcWithExtendedBrandAndStatusKeywords,
        DlcWithExtendedBrandAndStatusKeywordsHelp,
        DlcWithExtendedBrandAndStatusKeywordsOptIn,
        DlcWithExtendedBrandAndStatusKeywordsOptOut,
        DlcWithExtendedBrandAndStatusLinks,
        DlcWithExtendedBrandAndStatusMnoBrandTier,
        DlcWithExtendedBrandAndStatusMnoTcrTier,
        DlcWithExtendedBrandAndStatusOptions,
        DlcWithExtendedBrandAndStatusUseCase,
        EnhancedContact,
        EnhancedContactItem,
        Error,
        ExtendedBrand,
        ExtendedBrandWithVetting,
        ExtendedRcsCampaign,
        GetConversationParams,
        GetDlcCampaignStatusResponseUpdates,
        GetTollFreeCampaignStatusResponseUpdates,
        LinkClickEvent,
        LocationShareAction,
        LocationShareActionData,
        Message,
        MessageContent,
        MessageEvent,
        MessageEventContent,
        MessageEventConversation,
        MessageEventDirection,
        MessageEventMmsContent,
        MessageEventRcsButtonData,
        MessageEventRcsCardsContent,
        MessageEventRcsCardsContentCardsItem,
        MessageEventRcsLocationData,
        MessageEventRcsMediaContent,
        MessageEventRcsTextContent,
        MessageEventSmsContent,
        MessageList,
        MessageMethodEnum,
        MessageProtocolEnum,
        MessageSchedule,
        MessageStatusEnum,
        MessageVolumeEnum,
        MessageWithReaction,
        MessageWithReactionDirection,
        MessagingProfileEnum,
        MmsContent,
        MmsValidationResponseSegments,
        MmsValidationResponseSegmentsValueItem,
        MmsValidationResult,
        NotFoundErrorBody,
        NumberFormat,
        OptInMethodEnum,
        OptionalBrandInfo,
        OptionalContact,
        OptionalContacts,
        Pagination,
        Phone,
        PhoneCapabilities,
        PhoneEnum,
        PhoneFeatureEnum,
        PhoneNumberCampaignAttachFailedItem,
        PhoneNumberCampaignAttachPhoneNumbersItem,
        PhoneNumberCampaignAttachPhoneNumbersItemCampaign,
        PhoneNumberCampaignDetachFailedItem,
        PhoneNumberCampaignDetachPhoneNumbersItem,
        PhoneNumberCampaignDetachPhoneNumbersItemCampaign,
        PhoneNumberCost,
        PhoneNumberDetails,
        PhoneNumberEnum,
        PhoneNumberRegion,
        PhoneNumberStatus,
        PhoneNumberStatusEnum,
        PinnacleFileUploadMetadata,
        PinnacleUrlConfig,
        ProfileStatusEnum,
        PurchasedNumber,
        RcsButtonCall,
        RcsButtonOpenUrl,
        RcsButtonOpenUrlWebviewMode,
        RcsButtonRequestUserLocation,
        RcsButtonScheduleEvent,
        RcsButtonSendLocation,
        RcsButtonSendLocationLatLong,
        RcsButtonTrigger,
        RcsCampaign,
        RcsCampaignSchemaAgent,
        RcsCampaignSchemaAgentEmailsItem,
        RcsCampaignSchemaAgentPhonesItem,
        RcsCampaignSchemaAgentWebsitesItem,
        RcsCampaignSchemaExtraAgent,
        RcsCampaignSchemaExtraAgentEmailsItem,
        RcsCampaignSchemaExtraAgentPhonesItem,
        RcsCampaignSchemaExtraAgentWebsitesItem,
        RcsCampaignSchemaExtraKeywords,
        RcsCampaignSchemaExtraKeywordsHelp,
        RcsCampaignSchemaExtraKeywordsOptIn,
        RcsCampaignSchemaExtraKeywordsOptOut,
        RcsCampaignSchemaExtraLinks,
        RcsCampaignSchemaExtraTraffic,
        RcsCampaignSchemaExtraUseCase,
        RcsCampaignSchemaKeywords,
        RcsCampaignSchemaKeywordsHelp,
        RcsCampaignSchemaKeywordsOptIn,
        RcsCampaignSchemaKeywordsOptOut,
        RcsCampaignSchemaLinks,
        RcsCampaignSchemaTraffic,
        RcsCampaignSchemaUseCase,
        RcsCampaignStatus,
        RcsCampaignUseCaseEnum,
        RcsCapabilitiesResult,
        RcsCapability,
        RcsCapabilityActions,
        RcsCapabilityCards,
        RcsCardsContent,
        RcsCardsContentCardsItem,
        RcsContent,
        RcsLinkResult,
        RcsMediaContent,
        RcsMediaDetailsContent,
        RcsMessagingTypeEnum,
        RcsValidateContent,
        RcsValidateContentMedia,
        RcsValidationResult,
        RcsWhitelistResponse,
        ReactionResult,
        RefreshedFile,
        RichButton,
        RichButton_Call,
        RichButton_OpenUrl,
        RichButton_RequestUserLocation,
        RichButton_ScheduleEvent,
        RichButton_SendLocation,
        RichButton_Trigger,
        RichCard,
        RichCards,
        RichCardsMessage,
        RichMediaMessage,
        RichMessage,
        RichStandaloneCardOptions,
        RichText,
        RichTextMessage,
        ScheduleCancelResult,
        ScheduledBlastDetails,
        ScheduledBlastResponseConfig,
        ScheduledMessage,
        ScheduledSendResponseConfig,
        SendRcsCardOptionsStandaloneCardImageAlignment,
        SendRcsCardOptionsStandaloneCardOrientation,
        SendRichCardsOptions,
        SendRichMessageOptions,
        SendSmsResponseSegments,
        SendSmsResponseSegmentsEncoding,
        SendTypingIndicatorResponse,
        Sender,
        SentMmsDetails,
        SentRichMessage,
        SentSmsDetails,
        ShortenedUrl,
        ShortenedUrlWithClickData,
        SmsContent,
        SmsValidationResponseSegments,
        SmsValidationResponseSegmentsGsm7,
        SmsValidationResponseSegmentsUtf16,
        SmsValidationResponseTotal,
        SmsValidationResult,
        SubUseCaseEnum,
        SubmissionResults,
        SuccessfulConversationUpdate,
        TollFreeCampaign,
        TollFreeCampaignSchemaKeywords,
        TollFreeCampaignSchemaKeywordsHelp,
        TollFreeCampaignSchemaKeywordsOptIn,
        TollFreeCampaignSchemaLinks,
        TollFreeCampaignSchemaOptIn,
        TollFreeCampaignSchemaOptions,
        TollFreeCampaignSchemaUseCase,
        TollFreeCampaignStatus,
        TollFreeCampaignUseCaseEnum,
        TollFreeCampaignWithExtendedBrandAndStatus,
        TollFreeStatusEnum,
        TollFreeWithExtendedBrandAndStatusKeywords,
        TollFreeWithExtendedBrandAndStatusKeywordsHelp,
        TollFreeWithExtendedBrandAndStatusKeywordsOptIn,
        TollFreeWithExtendedBrandAndStatusLinks,
        TollFreeWithExtendedBrandAndStatusOptIn,
        TollFreeWithExtendedBrandAndStatusOptions,
        TollFreeWithExtendedBrandAndStatusUseCase,
        Tracking,
        UpdatedContactId,
        UploadResults,
        UserEvent,
        UserEventConversation,
        VCardAddressSchemaTypeItem,
        VCardData,
        VCardDataFile,
        VCardDataFileMetadata,
        VCardEmailSchemaTypeItem,
        VCardGeo,
        VCardName,
        VCardOrganization,
        VCardPhoneSchemaTypeItem,
        ValidateCampaignParams,
        ValidationErrorDetails,
        ValidationResults,
        Vcard,
        VcardAddress,
        VcardContent,
        VcardEmail,
        VcardPhone,
        VcardResource,
        VettingFeedback,
        VettingHistory,
        VettingHistoryVettingStatus,
        VettingResults,
        WebhookEventEnum,
        WebhookResult,
        Webhooks,
        ZodError,
    )
    from .errors import (
        BadRequestError,
        InternalServerError,
        NotFoundError,
        NotImplementedError,
        PaymentRequiredError,
        UnauthorizedError,
    )
    from . import (
        audiences,
        brands,
        campaigns,
        contacts,
        conversations,
        messages,
        phone_numbers,
        rcs,
        status,
        tools,
        webhooks,
    )
    from .audiences import AudiencesGetResponse
    from .brands import (
        AutofillBrandOptions,
        UpsertBrandSchemaContact,
        UpsertBrandSchemaEntityType,
        UpsertBrandSchemaSector,
        UpsertBrandSchemaType,
    )
    from .client import AsyncPinnacle, Pinnacle
    from .conversations import (
        ConversationsListMessagesRequestDirection,
        ConversationsListMessagesRequestSortOrder,
        ConversationsListMessagesRequestStatus,
        ConversationsListMessagesRequestType,
    )
    from .environment import PinnacleEnvironment
    from .messages import ReactMessageOptions
    from .phone_numbers import (
        EnhancedContactInfo,
        PhoneDetailsSchemaLevel,
        PhoneNumbersGetResponse,
        RetrievePhoneNumberDetailsOptions,
        SearchPhoneNumberByDigits,
        SearchPhoneNumberByLocation,
        SearchPhoneNumberOptions,
    )
    from .version import __version__
_dynamic_imports: typing.Dict[str, str] = {
    "AdvancedPhoneInformation": ".types",
    "AdvancedPhoneInformationCarrier": ".types",
    "AdvancedPhoneInformationContact": ".types",
    "AdvancedPhoneInformationLocation": ".types",
    "AdvancedPhoneInformationLocationCoordinates": ".types",
    "AdvancedPhoneInformationLocationCountry": ".types",
    "AdvancedPhoneInformationType": ".types",
    "AdvancedPhoneInformationTypeRecommendation": ".types",
    "Agent": ".types",
    "AsyncPinnacle": ".client",
    "AttachWebhookByIdParams": ".types",
    "AttachWebhookParams": ".types",
    "AttachWebhookResponseWebhook": ".types",
    "AttachedPhoneNumberResult": ".types",
    "AudienceCountOnly": ".types",
    "AudienceWithPagination": ".types",
    "AudiencesGetResponse": ".audiences",
    "AutofillBrandOptions": ".brands",
    "AutofillCampaignParams": ".types",
    "AutofillDlcCampaignResponse": ".types",
    "AutofillDlcResponseKeywords": ".types",
    "AutofillDlcResponseKeywordsHelp": ".types",
    "AutofillDlcResponseKeywordsOptIn": ".types",
    "AutofillDlcResponseKeywordsOptOut": ".types",
    "AutofillDlcResponseLinks": ".types",
    "AutofillDlcResponseOptions": ".types",
    "AutofillDlcResponseUseCase": ".types",
    "BadRequestError": ".errors",
    "BadRequestErrorBody": ".types",
    "BaseRichMessage": ".types",
    "BasicPhoneInformation": ".types",
    "BasicPhoneInformationContact": ".types",
    "BasicPhoneInformationLocation": ".types",
    "BasicPhoneInformationLocationCountry": ".types",
    "BlastDetails": ".types",
    "BrandStatus": ".types",
    "BrandStatusEnum": ".types",
    "ButtonClicked": ".types",
    "ButtonClickedData": ".types",
    "ButtonClickedDataButton": ".types",
    "ButtonClickedDataButtonRaw": ".types",
    "ButtonClickedDataButtonType": ".types",
    "BuyResponseCapabilities": ".types",
    "CampaignEnum": ".types",
    "CampaignQuery": ".types",
    "CampaignSubmissionResult": ".types",
    "CampaignValidationResponseErrorsItem": ".types",
    "CampaignValidationResult": ".types",
    "CompanyEntityTypeEnum": ".types",
    "CompanySectorEnum": ".types",
    "CompanyTypeEnum": ".types",
    "ConfiguredWebhook": ".types",
    "Contact": ".types",
    "ContactId": ".types",
    "Conversation": ".types",
    "ConversationByIdParams": ".types",
    "ConversationByParticipantsParams": ".types",
    "ConversationContact": ".types",
    "ConversationList": ".types",
    "ConversationSender": ".types",
    "ConversationsListMessagesRequestDirection": ".conversations",
    "ConversationsListMessagesRequestSortOrder": ".conversations",
    "ConversationsListMessagesRequestStatus": ".conversations",
    "ConversationsListMessagesRequestType": ".conversations",
    "CreateAndAttachWebhookByUrlParams": ".types",
    "CreateUrlOptions": ".types",
    "DeleteAudienceResponse": ".types",
    "DetachedPhoneNumberResult": ".types",
    "DetachedWebhookInfo": ".types",
    "DetailedPhoneNumberEnum": ".types",
    "DlcAssignmentStatusEnum": ".types",
    "DlcCampaignStatus": ".types",
    "DlcCampaignUseCaseEnum": ".types",
    "DlcCampaignWithExtendedBrandAndStatus": ".types",
    "DlcWithExtendedBrandAndStatusKeywords": ".types",
    "DlcWithExtendedBrandAndStatusKeywordsHelp": ".types",
    "DlcWithExtendedBrandAndStatusKeywordsOptIn": ".types",
    "DlcWithExtendedBrandAndStatusKeywordsOptOut": ".types",
    "DlcWithExtendedBrandAndStatusLinks": ".types",
    "DlcWithExtendedBrandAndStatusMnoBrandTier": ".types",
    "DlcWithExtendedBrandAndStatusMnoTcrTier": ".types",
    "DlcWithExtendedBrandAndStatusOptions": ".types",
    "DlcWithExtendedBrandAndStatusUseCase": ".types",
    "EnhancedContact": ".types",
    "EnhancedContactInfo": ".phone_numbers",
    "EnhancedContactItem": ".types",
    "Error": ".types",
    "ExtendedBrand": ".types",
    "ExtendedBrandWithVetting": ".types",
    "ExtendedRcsCampaign": ".types",
    "GetConversationParams": ".types",
    "GetDlcCampaignStatusResponseUpdates": ".types",
    "GetTollFreeCampaignStatusResponseUpdates": ".types",
    "InternalServerError": ".errors",
    "LinkClickEvent": ".types",
    "LocationShareAction": ".types",
    "LocationShareActionData": ".types",
    "Message": ".types",
    "MessageContent": ".types",
    "MessageEvent": ".types",
    "MessageEventContent": ".types",
    "MessageEventConversation": ".types",
    "MessageEventDirection": ".types",
    "MessageEventMmsContent": ".types",
    "MessageEventRcsButtonData": ".types",
    "MessageEventRcsCardsContent": ".types",
    "MessageEventRcsCardsContentCardsItem": ".types",
    "MessageEventRcsLocationData": ".types",
    "MessageEventRcsMediaContent": ".types",
    "MessageEventRcsTextContent": ".types",
    "MessageEventSmsContent": ".types",
    "MessageList": ".types",
    "MessageMethodEnum": ".types",
    "MessageProtocolEnum": ".types",
    "MessageSchedule": ".types",
    "MessageStatusEnum": ".types",
    "MessageVolumeEnum": ".types",
    "MessageWithReaction": ".types",
    "MessageWithReactionDirection": ".types",
    "MessagingProfileEnum": ".types",
    "MmsContent": ".types",
    "MmsValidationResponseSegments": ".types",
    "MmsValidationResponseSegmentsValueItem": ".types",
    "MmsValidationResult": ".types",
    "NotFoundError": ".errors",
    "NotFoundErrorBody": ".types",
    "NotImplementedError": ".errors",
    "NumberFormat": ".types",
    "OptInMethodEnum": ".types",
    "OptionalBrandInfo": ".types",
    "OptionalContact": ".types",
    "OptionalContacts": ".types",
    "Pagination": ".types",
    "PaymentRequiredError": ".errors",
    "Phone": ".types",
    "PhoneCapabilities": ".types",
    "PhoneDetailsSchemaLevel": ".phone_numbers",
    "PhoneEnum": ".types",
    "PhoneFeatureEnum": ".types",
    "PhoneNumberCampaignAttachFailedItem": ".types",
    "PhoneNumberCampaignAttachPhoneNumbersItem": ".types",
    "PhoneNumberCampaignAttachPhoneNumbersItemCampaign": ".types",
    "PhoneNumberCampaignDetachFailedItem": ".types",
    "PhoneNumberCampaignDetachPhoneNumbersItem": ".types",
    "PhoneNumberCampaignDetachPhoneNumbersItemCampaign": ".types",
    "PhoneNumberCost": ".types",
    "PhoneNumberDetails": ".types",
    "PhoneNumberEnum": ".types",
    "PhoneNumberRegion": ".types",
    "PhoneNumberStatus": ".types",
    "PhoneNumberStatusEnum": ".types",
    "PhoneNumbersGetResponse": ".phone_numbers",
    "Pinnacle": ".client",
    "PinnacleEnvironment": ".environment",
    "PinnacleFileUploadMetadata": ".types",
    "PinnacleUrlConfig": ".types",
    "ProfileStatusEnum": ".types",
    "PurchasedNumber": ".types",
    "RcsButtonCall": ".types",
    "RcsButtonOpenUrl": ".types",
    "RcsButtonOpenUrlWebviewMode": ".types",
    "RcsButtonRequestUserLocation": ".types",
    "RcsButtonScheduleEvent": ".types",
    "RcsButtonSendLocation": ".types",
    "RcsButtonSendLocationLatLong": ".types",
    "RcsButtonTrigger": ".types",
    "RcsCampaign": ".types",
    "RcsCampaignSchemaAgent": ".types",
    "RcsCampaignSchemaAgentEmailsItem": ".types",
    "RcsCampaignSchemaAgentPhonesItem": ".types",
    "RcsCampaignSchemaAgentWebsitesItem": ".types",
    "RcsCampaignSchemaExtraAgent": ".types",
    "RcsCampaignSchemaExtraAgentEmailsItem": ".types",
    "RcsCampaignSchemaExtraAgentPhonesItem": ".types",
    "RcsCampaignSchemaExtraAgentWebsitesItem": ".types",
    "RcsCampaignSchemaExtraKeywords": ".types",
    "RcsCampaignSchemaExtraKeywordsHelp": ".types",
    "RcsCampaignSchemaExtraKeywordsOptIn": ".types",
    "RcsCampaignSchemaExtraKeywordsOptOut": ".types",
    "RcsCampaignSchemaExtraLinks": ".types",
    "RcsCampaignSchemaExtraTraffic": ".types",
    "RcsCampaignSchemaExtraUseCase": ".types",
    "RcsCampaignSchemaKeywords": ".types",
    "RcsCampaignSchemaKeywordsHelp": ".types",
    "RcsCampaignSchemaKeywordsOptIn": ".types",
    "RcsCampaignSchemaKeywordsOptOut": ".types",
    "RcsCampaignSchemaLinks": ".types",
    "RcsCampaignSchemaTraffic": ".types",
    "RcsCampaignSchemaUseCase": ".types",
    "RcsCampaignStatus": ".types",
    "RcsCampaignUseCaseEnum": ".types",
    "RcsCapabilitiesResult": ".types",
    "RcsCapability": ".types",
    "RcsCapabilityActions": ".types",
    "RcsCapabilityCards": ".types",
    "RcsCardsContent": ".types",
    "RcsCardsContentCardsItem": ".types",
    "RcsContent": ".types",
    "RcsLinkResult": ".types",
    "RcsMediaContent": ".types",
    "RcsMediaDetailsContent": ".types",
    "RcsMessagingTypeEnum": ".types",
    "RcsValidateContent": ".types",
    "RcsValidateContentMedia": ".types",
    "RcsValidationResult": ".types",
    "RcsWhitelistResponse": ".types",
    "ReactMessageOptions": ".messages",
    "ReactionResult": ".types",
    "RefreshedFile": ".types",
    "RetrievePhoneNumberDetailsOptions": ".phone_numbers",
    "RichButton": ".types",
    "RichButton_Call": ".types",
    "RichButton_OpenUrl": ".types",
    "RichButton_RequestUserLocation": ".types",
    "RichButton_ScheduleEvent": ".types",
    "RichButton_SendLocation": ".types",
    "RichButton_Trigger": ".types",
    "RichCard": ".types",
    "RichCards": ".types",
    "RichCardsMessage": ".types",
    "RichMediaMessage": ".types",
    "RichMessage": ".types",
    "RichStandaloneCardOptions": ".types",
    "RichText": ".types",
    "RichTextMessage": ".types",
    "ScheduleCancelResult": ".types",
    "ScheduledBlastDetails": ".types",
    "ScheduledBlastResponseConfig": ".types",
    "ScheduledMessage": ".types",
    "ScheduledSendResponseConfig": ".types",
    "SearchPhoneNumberByDigits": ".phone_numbers",
    "SearchPhoneNumberByLocation": ".phone_numbers",
    "SearchPhoneNumberOptions": ".phone_numbers",
    "SendRcsCardOptionsStandaloneCardImageAlignment": ".types",
    "SendRcsCardOptionsStandaloneCardOrientation": ".types",
    "SendRichCardsOptions": ".types",
    "SendRichMessageOptions": ".types",
    "SendSmsResponseSegments": ".types",
    "SendSmsResponseSegmentsEncoding": ".types",
    "SendTypingIndicatorResponse": ".types",
    "Sender": ".types",
    "SentMmsDetails": ".types",
    "SentRichMessage": ".types",
    "SentSmsDetails": ".types",
    "ShortenedUrl": ".types",
    "ShortenedUrlWithClickData": ".types",
    "SmsContent": ".types",
    "SmsValidationResponseSegments": ".types",
    "SmsValidationResponseSegmentsGsm7": ".types",
    "SmsValidationResponseSegmentsUtf16": ".types",
    "SmsValidationResponseTotal": ".types",
    "SmsValidationResult": ".types",
    "SubUseCaseEnum": ".types",
    "SubmissionResults": ".types",
    "SuccessfulConversationUpdate": ".types",
    "TollFreeCampaign": ".types",
    "TollFreeCampaignSchemaKeywords": ".types",
    "TollFreeCampaignSchemaKeywordsHelp": ".types",
    "TollFreeCampaignSchemaKeywordsOptIn": ".types",
    "TollFreeCampaignSchemaLinks": ".types",
    "TollFreeCampaignSchemaOptIn": ".types",
    "TollFreeCampaignSchemaOptions": ".types",
    "TollFreeCampaignSchemaUseCase": ".types",
    "TollFreeCampaignStatus": ".types",
    "TollFreeCampaignUseCaseEnum": ".types",
    "TollFreeCampaignWithExtendedBrandAndStatus": ".types",
    "TollFreeStatusEnum": ".types",
    "TollFreeWithExtendedBrandAndStatusKeywords": ".types",
    "TollFreeWithExtendedBrandAndStatusKeywordsHelp": ".types",
    "TollFreeWithExtendedBrandAndStatusKeywordsOptIn": ".types",
    "TollFreeWithExtendedBrandAndStatusLinks": ".types",
    "TollFreeWithExtendedBrandAndStatusOptIn": ".types",
    "TollFreeWithExtendedBrandAndStatusOptions": ".types",
    "TollFreeWithExtendedBrandAndStatusUseCase": ".types",
    "Tracking": ".types",
    "UnauthorizedError": ".errors",
    "UpdatedContactId": ".types",
    "UploadResults": ".types",
    "UpsertBrandSchemaContact": ".brands",
    "UpsertBrandSchemaEntityType": ".brands",
    "UpsertBrandSchemaSector": ".brands",
    "UpsertBrandSchemaType": ".brands",
    "UserEvent": ".types",
    "UserEventConversation": ".types",
    "VCardAddressSchemaTypeItem": ".types",
    "VCardData": ".types",
    "VCardDataFile": ".types",
    "VCardDataFileMetadata": ".types",
    "VCardEmailSchemaTypeItem": ".types",
    "VCardGeo": ".types",
    "VCardName": ".types",
    "VCardOrganization": ".types",
    "VCardPhoneSchemaTypeItem": ".types",
    "ValidateCampaignParams": ".types",
    "ValidationErrorDetails": ".types",
    "ValidationResults": ".types",
    "Vcard": ".types",
    "VcardAddress": ".types",
    "VcardContent": ".types",
    "VcardEmail": ".types",
    "VcardPhone": ".types",
    "VcardResource": ".types",
    "VettingFeedback": ".types",
    "VettingHistory": ".types",
    "VettingHistoryVettingStatus": ".types",
    "VettingResults": ".types",
    "WebhookEventEnum": ".types",
    "WebhookResult": ".types",
    "Webhooks": ".types",
    "ZodError": ".types",
    "__version__": ".version",
    "audiences": ".",
    "brands": ".",
    "campaigns": ".",
    "contacts": ".",
    "conversations": ".",
    "messages": ".",
    "phone_numbers": ".",
    "rcs": ".",
    "status": ".",
    "tools": ".",
    "webhooks": ".",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AdvancedPhoneInformation",
    "AdvancedPhoneInformationCarrier",
    "AdvancedPhoneInformationContact",
    "AdvancedPhoneInformationLocation",
    "AdvancedPhoneInformationLocationCoordinates",
    "AdvancedPhoneInformationLocationCountry",
    "AdvancedPhoneInformationType",
    "AdvancedPhoneInformationTypeRecommendation",
    "Agent",
    "AsyncPinnacle",
    "AttachWebhookByIdParams",
    "AttachWebhookParams",
    "AttachWebhookResponseWebhook",
    "AttachedPhoneNumberResult",
    "AudienceCountOnly",
    "AudienceWithPagination",
    "AudiencesGetResponse",
    "AutofillBrandOptions",
    "AutofillCampaignParams",
    "AutofillDlcCampaignResponse",
    "AutofillDlcResponseKeywords",
    "AutofillDlcResponseKeywordsHelp",
    "AutofillDlcResponseKeywordsOptIn",
    "AutofillDlcResponseKeywordsOptOut",
    "AutofillDlcResponseLinks",
    "AutofillDlcResponseOptions",
    "AutofillDlcResponseUseCase",
    "BadRequestError",
    "BadRequestErrorBody",
    "BaseRichMessage",
    "BasicPhoneInformation",
    "BasicPhoneInformationContact",
    "BasicPhoneInformationLocation",
    "BasicPhoneInformationLocationCountry",
    "BlastDetails",
    "BrandStatus",
    "BrandStatusEnum",
    "ButtonClicked",
    "ButtonClickedData",
    "ButtonClickedDataButton",
    "ButtonClickedDataButtonRaw",
    "ButtonClickedDataButtonType",
    "BuyResponseCapabilities",
    "CampaignEnum",
    "CampaignQuery",
    "CampaignSubmissionResult",
    "CampaignValidationResponseErrorsItem",
    "CampaignValidationResult",
    "CompanyEntityTypeEnum",
    "CompanySectorEnum",
    "CompanyTypeEnum",
    "ConfiguredWebhook",
    "Contact",
    "ContactId",
    "Conversation",
    "ConversationByIdParams",
    "ConversationByParticipantsParams",
    "ConversationContact",
    "ConversationList",
    "ConversationSender",
    "ConversationsListMessagesRequestDirection",
    "ConversationsListMessagesRequestSortOrder",
    "ConversationsListMessagesRequestStatus",
    "ConversationsListMessagesRequestType",
    "CreateAndAttachWebhookByUrlParams",
    "CreateUrlOptions",
    "DeleteAudienceResponse",
    "DetachedPhoneNumberResult",
    "DetachedWebhookInfo",
    "DetailedPhoneNumberEnum",
    "DlcAssignmentStatusEnum",
    "DlcCampaignStatus",
    "DlcCampaignUseCaseEnum",
    "DlcCampaignWithExtendedBrandAndStatus",
    "DlcWithExtendedBrandAndStatusKeywords",
    "DlcWithExtendedBrandAndStatusKeywordsHelp",
    "DlcWithExtendedBrandAndStatusKeywordsOptIn",
    "DlcWithExtendedBrandAndStatusKeywordsOptOut",
    "DlcWithExtendedBrandAndStatusLinks",
    "DlcWithExtendedBrandAndStatusMnoBrandTier",
    "DlcWithExtendedBrandAndStatusMnoTcrTier",
    "DlcWithExtendedBrandAndStatusOptions",
    "DlcWithExtendedBrandAndStatusUseCase",
    "EnhancedContact",
    "EnhancedContactInfo",
    "EnhancedContactItem",
    "Error",
    "ExtendedBrand",
    "ExtendedBrandWithVetting",
    "ExtendedRcsCampaign",
    "GetConversationParams",
    "GetDlcCampaignStatusResponseUpdates",
    "GetTollFreeCampaignStatusResponseUpdates",
    "InternalServerError",
    "LinkClickEvent",
    "LocationShareAction",
    "LocationShareActionData",
    "Message",
    "MessageContent",
    "MessageEvent",
    "MessageEventContent",
    "MessageEventConversation",
    "MessageEventDirection",
    "MessageEventMmsContent",
    "MessageEventRcsButtonData",
    "MessageEventRcsCardsContent",
    "MessageEventRcsCardsContentCardsItem",
    "MessageEventRcsLocationData",
    "MessageEventRcsMediaContent",
    "MessageEventRcsTextContent",
    "MessageEventSmsContent",
    "MessageList",
    "MessageMethodEnum",
    "MessageProtocolEnum",
    "MessageSchedule",
    "MessageStatusEnum",
    "MessageVolumeEnum",
    "MessageWithReaction",
    "MessageWithReactionDirection",
    "MessagingProfileEnum",
    "MmsContent",
    "MmsValidationResponseSegments",
    "MmsValidationResponseSegmentsValueItem",
    "MmsValidationResult",
    "NotFoundError",
    "NotFoundErrorBody",
    "NotImplementedError",
    "NumberFormat",
    "OptInMethodEnum",
    "OptionalBrandInfo",
    "OptionalContact",
    "OptionalContacts",
    "Pagination",
    "PaymentRequiredError",
    "Phone",
    "PhoneCapabilities",
    "PhoneDetailsSchemaLevel",
    "PhoneEnum",
    "PhoneFeatureEnum",
    "PhoneNumberCampaignAttachFailedItem",
    "PhoneNumberCampaignAttachPhoneNumbersItem",
    "PhoneNumberCampaignAttachPhoneNumbersItemCampaign",
    "PhoneNumberCampaignDetachFailedItem",
    "PhoneNumberCampaignDetachPhoneNumbersItem",
    "PhoneNumberCampaignDetachPhoneNumbersItemCampaign",
    "PhoneNumberCost",
    "PhoneNumberDetails",
    "PhoneNumberEnum",
    "PhoneNumberRegion",
    "PhoneNumberStatus",
    "PhoneNumberStatusEnum",
    "PhoneNumbersGetResponse",
    "Pinnacle",
    "PinnacleEnvironment",
    "PinnacleFileUploadMetadata",
    "PinnacleUrlConfig",
    "ProfileStatusEnum",
    "PurchasedNumber",
    "RcsButtonCall",
    "RcsButtonOpenUrl",
    "RcsButtonOpenUrlWebviewMode",
    "RcsButtonRequestUserLocation",
    "RcsButtonScheduleEvent",
    "RcsButtonSendLocation",
    "RcsButtonSendLocationLatLong",
    "RcsButtonTrigger",
    "RcsCampaign",
    "RcsCampaignSchemaAgent",
    "RcsCampaignSchemaAgentEmailsItem",
    "RcsCampaignSchemaAgentPhonesItem",
    "RcsCampaignSchemaAgentWebsitesItem",
    "RcsCampaignSchemaExtraAgent",
    "RcsCampaignSchemaExtraAgentEmailsItem",
    "RcsCampaignSchemaExtraAgentPhonesItem",
    "RcsCampaignSchemaExtraAgentWebsitesItem",
    "RcsCampaignSchemaExtraKeywords",
    "RcsCampaignSchemaExtraKeywordsHelp",
    "RcsCampaignSchemaExtraKeywordsOptIn",
    "RcsCampaignSchemaExtraKeywordsOptOut",
    "RcsCampaignSchemaExtraLinks",
    "RcsCampaignSchemaExtraTraffic",
    "RcsCampaignSchemaExtraUseCase",
    "RcsCampaignSchemaKeywords",
    "RcsCampaignSchemaKeywordsHelp",
    "RcsCampaignSchemaKeywordsOptIn",
    "RcsCampaignSchemaKeywordsOptOut",
    "RcsCampaignSchemaLinks",
    "RcsCampaignSchemaTraffic",
    "RcsCampaignSchemaUseCase",
    "RcsCampaignStatus",
    "RcsCampaignUseCaseEnum",
    "RcsCapabilitiesResult",
    "RcsCapability",
    "RcsCapabilityActions",
    "RcsCapabilityCards",
    "RcsCardsContent",
    "RcsCardsContentCardsItem",
    "RcsContent",
    "RcsLinkResult",
    "RcsMediaContent",
    "RcsMediaDetailsContent",
    "RcsMessagingTypeEnum",
    "RcsValidateContent",
    "RcsValidateContentMedia",
    "RcsValidationResult",
    "RcsWhitelistResponse",
    "ReactMessageOptions",
    "ReactionResult",
    "RefreshedFile",
    "RetrievePhoneNumberDetailsOptions",
    "RichButton",
    "RichButton_Call",
    "RichButton_OpenUrl",
    "RichButton_RequestUserLocation",
    "RichButton_ScheduleEvent",
    "RichButton_SendLocation",
    "RichButton_Trigger",
    "RichCard",
    "RichCards",
    "RichCardsMessage",
    "RichMediaMessage",
    "RichMessage",
    "RichStandaloneCardOptions",
    "RichText",
    "RichTextMessage",
    "ScheduleCancelResult",
    "ScheduledBlastDetails",
    "ScheduledBlastResponseConfig",
    "ScheduledMessage",
    "ScheduledSendResponseConfig",
    "SearchPhoneNumberByDigits",
    "SearchPhoneNumberByLocation",
    "SearchPhoneNumberOptions",
    "SendRcsCardOptionsStandaloneCardImageAlignment",
    "SendRcsCardOptionsStandaloneCardOrientation",
    "SendRichCardsOptions",
    "SendRichMessageOptions",
    "SendSmsResponseSegments",
    "SendSmsResponseSegmentsEncoding",
    "SendTypingIndicatorResponse",
    "Sender",
    "SentMmsDetails",
    "SentRichMessage",
    "SentSmsDetails",
    "ShortenedUrl",
    "ShortenedUrlWithClickData",
    "SmsContent",
    "SmsValidationResponseSegments",
    "SmsValidationResponseSegmentsGsm7",
    "SmsValidationResponseSegmentsUtf16",
    "SmsValidationResponseTotal",
    "SmsValidationResult",
    "SubUseCaseEnum",
    "SubmissionResults",
    "SuccessfulConversationUpdate",
    "TollFreeCampaign",
    "TollFreeCampaignSchemaKeywords",
    "TollFreeCampaignSchemaKeywordsHelp",
    "TollFreeCampaignSchemaKeywordsOptIn",
    "TollFreeCampaignSchemaLinks",
    "TollFreeCampaignSchemaOptIn",
    "TollFreeCampaignSchemaOptions",
    "TollFreeCampaignSchemaUseCase",
    "TollFreeCampaignStatus",
    "TollFreeCampaignUseCaseEnum",
    "TollFreeCampaignWithExtendedBrandAndStatus",
    "TollFreeStatusEnum",
    "TollFreeWithExtendedBrandAndStatusKeywords",
    "TollFreeWithExtendedBrandAndStatusKeywordsHelp",
    "TollFreeWithExtendedBrandAndStatusKeywordsOptIn",
    "TollFreeWithExtendedBrandAndStatusLinks",
    "TollFreeWithExtendedBrandAndStatusOptIn",
    "TollFreeWithExtendedBrandAndStatusOptions",
    "TollFreeWithExtendedBrandAndStatusUseCase",
    "Tracking",
    "UnauthorizedError",
    "UpdatedContactId",
    "UploadResults",
    "UpsertBrandSchemaContact",
    "UpsertBrandSchemaEntityType",
    "UpsertBrandSchemaSector",
    "UpsertBrandSchemaType",
    "UserEvent",
    "UserEventConversation",
    "VCardAddressSchemaTypeItem",
    "VCardData",
    "VCardDataFile",
    "VCardDataFileMetadata",
    "VCardEmailSchemaTypeItem",
    "VCardGeo",
    "VCardName",
    "VCardOrganization",
    "VCardPhoneSchemaTypeItem",
    "ValidateCampaignParams",
    "ValidationErrorDetails",
    "ValidationResults",
    "Vcard",
    "VcardAddress",
    "VcardContent",
    "VcardEmail",
    "VcardPhone",
    "VcardResource",
    "VettingFeedback",
    "VettingHistory",
    "VettingHistoryVettingStatus",
    "VettingResults",
    "WebhookEventEnum",
    "WebhookResult",
    "Webhooks",
    "ZodError",
    "__version__",
    "audiences",
    "brands",
    "campaigns",
    "contacts",
    "conversations",
    "messages",
    "phone_numbers",
    "rcs",
    "status",
    "tools",
    "webhooks",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class MessageEventConversation(UniversalBaseModel):
    """
    Conversation metadata containing the conversation ID, sender, and recipient information.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the conversation. This identifier is a string that always begins with the prefix `conv_`, for example: `conv_1234567890`. <br><br>
    To get more conversation details, use the [POST /conversations/get](/api-reference/conversations/get) endpoint.
    """

    from_: typing_extensions.Annotated[str, FieldMetadata(alias="from")] = pydantic.Field()
    """
    Sender's phone number or agent ID.
    """

    to: str = pydantic.Field()
    """
    Recipient's phone number.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .button_clicked_data import ButtonClickedData
from .location_share_action import LocationShareAction

ButtonClicked = typing.Union[ButtonClickedData, LocationShareAction]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.audience_count_only import AudienceCountOnly
from .raw_client import AsyncRawContactsClient, RawContactsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ContactsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawContactsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawContactsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawContactsClient
        """
        return self._raw_client

    def remove(
        self, *, id: str, contacts: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AudienceCountOnly:
        """
        Remove contacts from an existing audience. This operation is idempotent.

        - Only removes contacts that exist in the audience
        - Contacts not in the audience are ignored

        Parameters
        ----------
        id : str
            Audience ID. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`.

        contacts : typing.Sequence[str]
            Array of phone numbers (E.164 format) or contact IDs. <br><br>
            **Limit:** 1 min

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AudienceCountOnly
            Successfully removed contacts from the audience.

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.audiences.contacts.remove(
            id="aud_abc123",
            contacts=["+12125551234", "co_def456"],
        )
        """
        _response = self._raw_client.remove(id=id, contacts=contacts, request_options=request_options)
        return _response.data

    def add(
        self, *, id: str, contacts: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AudienceCountOnly:
        """
        Add contacts to an existing audience. This operation is additive and idempotent.

        - Phone numbers that don't exist will be auto-created as contacts
        - Duplicate adds are ignored
        - Contacts already in the audience are ignored

        Parameters
        ----------
        id : str
            Audience ID. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`.

        contacts : typing.Sequence[str]
            Array of phone numbers (E.164 format) or contact IDs. <br><br>
            **Limit:** 1 min

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AudienceCountOnly
            Successfully added contacts to the audience.

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.audiences.contacts.add(
            id="aud_abc123",
            contacts=["+12125551234", "co_def456", "+13105551234"],
        )
        """
        _response = self._raw_client.add(id=id, contacts=contacts, request_options=request_options)
        return _response.data


class AsyncContactsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawContactsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawContactsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawContactsClient
        """
        return self._raw_client

    async def remove(
        self, *, id: str, contacts: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AudienceCountOnly:
        """
        Remove contacts from an existing audience. This operation is idempotent.

        - Only removes contacts that exist in the audience
        - Contacts not in the audience are ignored

        Parameters
        ----------
        id : str
            Audience ID. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`.

        contacts : typing.Sequence[str]
            Array of phone numbers (E.164 format) or contact IDs. <br><br>
            **Limit:** 1 min

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AudienceCountOnly
            Successfully removed contacts from the audience.

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.audiences.contacts.remove(
                id="aud_abc123",
                contacts=["+12125551234", "co_def456"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.remove(id=id, contacts=contacts, request_options=request_options)
        return _response.data

    async def add(
        self, *, id: str, contacts: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AudienceCountOnly:
        """
        Add contacts to an existing audience. This operation is additive and idempotent.

        - Phone numbers that don't exist will be auto-created as contacts
        - Duplicate adds are ignored
        - Contacts already in the audience are ignored

        Parameters
        ----------
        id : str
            Audience ID. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`.

        contacts : typing.Sequence[str]
            Array of phone numbers (E.164 format) or contact IDs. <br><br>
            **Limit:** 1 min

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AudienceCountOnly
            Successfully added contacts to the audience.

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.audiences.contacts.add(
                id="aud_abc123",
                contacts=["+12125551234", "co_def456", "+13105551234"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add(id=id, contacts=contacts, request_options=request_options)
        return _response.data



import typing
from .base_client import PinnacleBase, AsyncPinnacleBase

if typing.TYPE_CHECKING:
    from .wrapper.messages.client import EnhancedMessages, AsyncEnhancedMessages
    from .wrapper.tools.client import EnhancedTools, AsyncEnhancedTools


class Pinnacle(PinnacleBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._enhanced_messages: typing.Optional["EnhancedMessages"] = None
        self._enhanced_tools: typing.Optional["EnhancedTools"] = None

    @property
    def messages(self) -> "EnhancedMessages":
        if self._enhanced_messages is None:
            from .wrapper.messages.client import EnhancedMessages  # noqa: E402

            self._enhanced_messages = EnhancedMessages(client_wrapper=self._client_wrapper)
        return self._enhanced_messages

    @property
    def tools(self) -> "EnhancedTools":
        if self._enhanced_tools is None:
            from .wrapper.tools.client import EnhancedTools  # noqa: E402

            self._enhanced_tools = EnhancedTools(client_wrapper=self._client_wrapper)
        return self._enhanced_tools


class AsyncPinnacle(AsyncPinnacleBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._enhanced_messages: typing.Optional["AsyncEnhancedMessages"] = None
        self._enhanced_tools: typing.Optional["AsyncEnhancedTools"] = None

    @property
    def messages(self) -> "AsyncEnhancedMessages":
        if self._enhanced_messages is None:
            from .wrapper.messages.client import AsyncEnhancedMessages  # noqa: E402

            self._enhanced_messages = AsyncEnhancedMessages(client_wrapper=self._client_wrapper)
        return self._enhanced_messages

    @property
    def tools(self) -> "AsyncEnhancedTools":
        if self._enhanced_tools is None:
            from .wrapper.tools.client import AsyncEnhancedTools  # noqa: E402

            self._enhanced_tools = AsyncEnhancedTools(client_wrapper=self._client_wrapper)
        return self._enhanced_tools



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .basic_phone_information_contact import BasicPhoneInformationContact
from .basic_phone_information_location import BasicPhoneInformationLocation
from .number_format import NumberFormat
from .phone_number_enum import PhoneNumberEnum


class BasicPhoneInformation(UniversalBaseModel):
    """
    Key details about a phone number, including its validity, type, location, carrier, and contact information.
    Provides the essential data required for verification, display, and basic analysis.
    """

    is_valid: typing_extensions.Annotated[bool, FieldMetadata(alias="isValid")] = pydantic.Field()
    """
    Indicates whether the phone number is valid and capable of receiving communications.
    """

    type: PhoneNumberEnum = pydantic.Field()
    """
    Classification of the phone number.
    """

    formats: NumberFormat = pydantic.Field()
    """
    Different standardized ways the phone number can be formatted for display.
    """

    location: BasicPhoneInformationLocation = pydantic.Field()
    """
    Geographic and political details where the phone number is registered.
    """

    carrier: str = pydantic.Field()
    """
    The telecommunications carrier or service provider for the number.
    """

    contact: BasicPhoneInformationContact = pydantic.Field()
    """
    Contact information linked to the phone number registration, if available.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .rcs_campaign_schema_agent import RcsCampaignSchemaAgent
from .rcs_campaign_schema_keywords import RcsCampaignSchemaKeywords
from .rcs_campaign_schema_links import RcsCampaignSchemaLinks
from .rcs_campaign_schema_traffic import RcsCampaignSchemaTraffic
from .rcs_campaign_schema_use_case import RcsCampaignSchemaUseCase
from .rcs_messaging_type_enum import RcsMessagingTypeEnum


class RcsCampaign(UniversalBaseModel):
    agent: typing.Optional[RcsCampaignSchemaAgent] = pydantic.Field(default=None)
    """
    Agent configured to the campaign.
    """

    campaign_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="campaignId")] = pydantic.Field(
        default=None
    )
    """
    Unique identifier for the campaign.
    """

    expected_agent_responses: typing_extensions.Annotated[
        typing.Optional[typing.List[str]], FieldMetadata(alias="expectedAgentResponses")
    ] = pydantic.Field(default=None)
    """
    List of what the agent might say to users. See the [Expected Agent Responses](/guides/campaigns/rcs-compliance#expected-agent-responses) section for requirements.
    """

    links: typing.Optional[RcsCampaignSchemaLinks] = pydantic.Field(default=None)
    """
    Legal documentation links.
    """

    use_case: typing_extensions.Annotated[typing.Optional[RcsCampaignSchemaUseCase], FieldMetadata(alias="useCase")] = (
        pydantic.Field(default=None)
    )
    """
    Use case classification for the campaign.
    """

    opt_in_terms_and_conditions: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="optInTermsAndConditions")
    ] = pydantic.Field(default=None)
    """
    Details on how opt-in is acquired. If it is done through a website or app, provide the link. See the [Opt-In Terms and Conditions](/guides/campaigns/rcs-compliance#opt-in-terms-and-conditions) section for requirements.
    """

    messaging_type: typing_extensions.Annotated[
        typing.Optional[RcsMessagingTypeEnum], FieldMetadata(alias="messagingType")
    ] = None
    carrier_description: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="carrierDescription")
    ] = pydantic.Field(default=None)
    """
    Description of the agent's purpose, shown to carriers for approval. See the [Carrier Description](/guides/campaigns/rcs-compliance#carrier-description) section for requirements.
    """

    keywords: typing.Optional[RcsCampaignSchemaKeywords] = None
    traffic: typing.Optional[RcsCampaignSchemaTraffic] = None
    agent_triggers: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="agentTriggers")] = (
        pydantic.Field(default=None)
    )
    """
    Explanation of how the agent is triggered. This includes how the first message is delivered, whether messages follow a schedule or triggered by user actions, and any external triggers. See the [Agent Triggers](/guides/campaigns/rcs-compliance#agent-triggers) section for requirements.
    """

    interaction_description: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="interactionDescription")
    ] = pydantic.Field(default=None)
    """
    Description of all agent interactions, including primary and secondary use cases. See the [Interaction Description](/guides/campaigns/rcs-compliance#interaction-description) section for requirements.
    """

    is_conversational: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="isConversational")] = (
        pydantic.Field(default=None)
    )
    """
    Whether the agent supports conversational flows or respond to P2A messages from the users. Set to false for one-way messages from agent to user.
    """

    cta_language: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ctaLanguage")] = (
        pydantic.Field(default=None)
    )
    """
    Required text that appears next to the opt-in checkbox for your opt-in form. This checkbox has to be unchecked by default. See the [CTA Language](/guides/campaigns/rcs-compliance#cta-language-opt-in-disclosure) section for requirements.
    """

    demo_trigger: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="demoTrigger")] = (
        pydantic.Field(default=None)
    )
    """
    Instructions on how an external reviewer can trigger messages and an example flow from the agent. This is usually an inbound text message to the agent that will start a flow of messages between the agent and the user. See the [Demo Trigger](/guides/campaigns/rcs-compliance#demo-trigger) section for requirements.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .rcs_campaign_schema_extra_keywords_help import RcsCampaignSchemaExtraKeywordsHelp
from .rcs_campaign_schema_extra_keywords_opt_in import RcsCampaignSchemaExtraKeywordsOptIn
from .rcs_campaign_schema_extra_keywords_opt_out import RcsCampaignSchemaExtraKeywordsOptOut


class RcsCampaignSchemaExtraKeywords(UniversalBaseModel):
    help: typing_extensions.Annotated[
        typing.Optional[RcsCampaignSchemaExtraKeywordsHelp], FieldMetadata(alias="HELP")
    ] = None
    opt_in: typing_extensions.Annotated[
        typing.Optional[RcsCampaignSchemaExtraKeywordsOptIn], FieldMetadata(alias="OPT_IN")
    ] = None
    opt_out: typing_extensions.Annotated[
        typing.Optional[RcsCampaignSchemaExtraKeywordsOptOut], FieldMetadata(alias="OPT_OUT")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class SendTypingIndicatorResponse(UniversalBaseModel):
    """
    Response schema confirming the typing indicator was sent.
    """

    success: bool = pydantic.Field()
    """
    Indicates whether the typing indicator was successfully sent.
    """

    agent_id: typing_extensions.Annotated[str, FieldMetadata(alias="agentId")] = pydantic.Field()
    """
    The RCS agent that sent the typing indicator.
    """

    recipient: str = pydantic.Field()
    """
    The recipient's phone number that received the typing indicator.
    """

    started_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="startedAt")] = pydantic.Field()
    """
    Timestamp when the typing indicator was started (ISO 8601 format).
    """

    ended_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="endedAt")] = pydantic.Field()
    """
    Timestamp when the typing indicator will automatically expire (ISO 8601 format). <br>
    
    This is typically one minute after `startedAt` unless a message is sent first.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

CompanySectorEnum = typing.Union[
    typing.Literal[
        "AGRICULTURE",
        "COMMUNICATION",
        "CONSTRUCTION",
        "EDUCATION",
        "ENERGY",
        "ENTERTAINMENT",
        "FINANCIAL",
        "GAMBLING",
        "GOVERNMENT",
        "HEALTHCARE",
        "HOSPITALITY",
        "INSURANCE",
        "MANUFACTURING",
        "NGO",
        "REAL_ESTATE",
        "RETAIL",
        "TECHNOLOGY",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .brand_status_enum import BrandStatusEnum


class BrandStatus(UniversalBaseModel):
    """
    Response for get brand status.
    """

    errors: typing.List[str] = pydantic.Field()
    """
    List of errors that occurred.
    """

    id: str = pydantic.Field()
    """
    The unique identifier of the brand. This identifier is a string that always begins with the prefix `b_`, for example: `b_1234567890`.
    """

    status: BrandStatusEnum = pydantic.Field()
    """
    Current review status of the brand. <br>
    
    `INCOMPLETE`: Has not been submitted. Use the [submit endpoint](../brands/submit). <br>
    `PENDING`: Currently being reviewed. <br>
    `VERIFIED`: Verified and good to go. <br>
    `UNVERIFIED`: No longer verified due to an update in the brand details. Requires vetting to be reverified. <br>
    `VETTED`: Brand has been vetted by an external provider and has received vetting scores. <br>
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ....core.serialization import FieldMetadata
from ....types.message_schedule import MessageSchedule
from ....types.tracking import Tracking


class SendMmsOptions(UniversalBaseModel):
    """
    Control how your MMS is processed and delivered.
    """

    multiple_messages: bool = pydantic.Field()
    """
    Allow splitting large content into multiple MMS messages if size limits are exceeded. <br>
    
    If `false`, MMS over the limit will throw an error and won't be sent.
    """

    schedule: typing.Optional[MessageSchedule] = None
    tracking: typing.Optional[Tracking] = None
    validate_: typing_extensions.Annotated[bool, FieldMetadata(alias="validate")] = pydantic.Field()
    """
    Set to true to check this message and throw an error when validation fails. <br>
    
    If errors occur, the message is not sent. Otherwise message is sent after validation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class SmsValidationResponseTotal(UniversalBaseModel):
    gsm_7: typing_extensions.Annotated[float, FieldMetadata(alias="gsm7")] = pydantic.Field()
    """
    Total price of this message. Calculated by multiplying the number of GSM-7 segments by the unit cost. <br>
    
    You must remove all unsupported UTF-16 characters before this message can be sent as GSM-7.
    """

    utf_16: typing_extensions.Annotated[float, FieldMetadata(alias="utf16")] = pydantic.Field()
    """
    Total price of this message. Calculated by multiplying the number of UTF-16 segments by the unit cost.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .messaging_profile_enum import MessagingProfileEnum


class PhoneNumberCampaignDetachPhoneNumbersItemCampaign(UniversalBaseModel):
    """
    Campaign that the phone is now detached from.
    """

    id: str = pydantic.Field()
    """
    Id of the campaign. For `TOLL_FREE` campaigns, it will begin with the prefix `tf_`, for example: `tf_1234567890`. For `10DLC` campaigns, it will begin with the prefix `dlc_`, for example: `dlc_1234567890`.
    """

    name: str = pydantic.Field()
    """
    Campaign's name.
    """

    type: MessagingProfileEnum

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .rich_button import RichButton


class RichCard(UniversalBaseModel):
    """
    Individual card containing title, optional media, and action buttons.
    """

    buttons: typing.List[RichButton] = pydantic.Field()
    """
    While buttons resemble quick replies, they remain on screen after being clicked and are card-specific. <br><br>
    **Limit:** 4 max
    """

    media: typing.Optional[str] = pydantic.Field(default=None)
    """
    Media file URLs to send.
    """

    subtitle: typing.Optional[str] = pydantic.Field(default=None)
    """
    Secondary text displayed below the title for additional context.
    """

    title: str = pydantic.Field()
    """
    Title displayed on the card.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .opt_in_method_enum import OptInMethodEnum


class TollFreeCampaignSchemaOptIn(UniversalBaseModel):
    """
    Opt-in method and workflow.
    """

    method: typing.Optional[OptInMethodEnum] = None
    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL where the opt-in form or documentation can be found.
    """

    workflow_description: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="workflowDescription")
    ] = pydantic.Field(default=None)
    """
    Describe your opt-in workflow. See the [Opt-In Methods and Workflow](/guides/campaigns/opt-in-compliance#opt-in-methods-and-workflow) section for requirements and examples.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConversationsListMessagesRequestStatus = typing.Union[
    typing.Literal["PENDING", "SENT", "SEND_FAILED", "DELIVERED", "DELIVERY_FAILED", "RECEIVED", "READ"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .rich_standalone_card_options import RichStandaloneCardOptions
from .send_rich_message_options import SendRichMessageOptions


class SendRichCardsOptions(SendRichMessageOptions):
    standalone_card: typing.Optional[RichStandaloneCardOptions] = pydantic.Field(default=None)
    """
    Configure standalone card layout options for enhanced visual presentation.
    
    > **⚠️ Important Restriction**
    >
    > This option is **only valid for single card messages** with static media. Using it with multiple cards will cause the request to fail with a validation error.
    > GIFs and videos are not supported and will be rendered as vertical cards.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .send_sms_response_segments_encoding import SendSmsResponseSegmentsEncoding


class SendSmsResponseSegments(UniversalBaseModel):
    """
    Details about how the message was segmented for sending.
    """

    count: float = pydantic.Field()
    """
    Number of message segments used to send this message.
    """

    encoding: SendSmsResponseSegmentsEncoding = pydantic.Field()
    """
    Encoding used for the message segments.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .rcs_validate_content_media import RcsValidateContentMedia
from .rich_cards import RichCards
from .rich_text import RichText

RcsValidateContent = typing.Union[RichText, RcsValidateContentMedia, RichCards]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.internal_server_error import InternalServerError
from ...errors.not_found_error import NotFoundError
from ...errors.not_implemented_error import NotImplementedError
from ...errors.payment_required_error import PaymentRequiredError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.error import Error
from ...types.mms_validation_result import MmsValidationResult
from .types.mms_send_response import MmsSendResponse
from .types.send_mms_options import SendMmsOptions

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawMmsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def send(
        self,
        *,
        from_: str,
        media_urls: typing.Sequence[str],
        text: str,
        to: str,
        options: typing.Optional[SendMmsOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[MmsSendResponse]:
        """
        Send a MMS immediately or schedule it for future delivery.

        Parameters
        ----------
        from_ : str
            Phone number you want to send the message from in E.164 format.

        media_urls : typing.Sequence[str]
            Media file URLs to send.<br>

             See [supported media types](https://app.pinnacle.sh/supported-file-types?type=MMS).

        text : str
            Message text to accompany the media.

        to : str
            Recipient's phone number in E.164 format.

        options : typing.Optional[SendMmsOptions]
            Control how your MMS is processed and delivered.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[MmsSendResponse]
            Successfully sent or scheduled the message. <br>

            Each message part can be tracked independently using its unique message ID. Use our [/messages/:id](./get) endpoint to track your messages.
        """
        _response = self._client_wrapper.httpx_client.request(
            "messages/send/mms",
            method="POST",
            json={
                "from": from_,
                "mediaUrls": media_urls,
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=SendMmsOptions, direction="write"
                ),
                "text": text,
                "to": to,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MmsSendResponse,
                    parse_obj_as(
                        type_=MmsSendResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def validate(
        self,
        *,
        media_urls: typing.Sequence[str],
        text: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[MmsValidationResult]:
        """
        Validate MMS message content without sending it.

        Parameters
        ----------
        media_urls : typing.Sequence[str]
            URLs of media files in this message. <br>

            See [supported media types](https://app.pinnacle.sh/supported-file-types?type=MMS).

        text : typing.Optional[str]
            Text content that accompanies the media files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[MmsValidationResult]
            Returns validation results.
        """
        _response = self._client_wrapper.httpx_client.request(
            "messages/validate/mms",
            method="POST",
            json={
                "mediaUrls": media_urls,
                "text": text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MmsValidationResult,
                    parse_obj_as(
                        type_=MmsValidationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawMmsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def send(
        self,
        *,
        from_: str,
        media_urls: typing.Sequence[str],
        text: str,
        to: str,
        options: typing.Optional[SendMmsOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[MmsSendResponse]:
        """
        Send a MMS immediately or schedule it for future delivery.

        Parameters
        ----------
        from_ : str
            Phone number you want to send the message from in E.164 format.

        media_urls : typing.Sequence[str]
            Media file URLs to send.<br>

             See [supported media types](https://app.pinnacle.sh/supported-file-types?type=MMS).

        text : str
            Message text to accompany the media.

        to : str
            Recipient's phone number in E.164 format.

        options : typing.Optional[SendMmsOptions]
            Control how your MMS is processed and delivered.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[MmsSendResponse]
            Successfully sent or scheduled the message. <br>

            Each message part can be tracked independently using its unique message ID. Use our [/messages/:id](./get) endpoint to track your messages.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "messages/send/mms",
            method="POST",
            json={
                "from": from_,
                "mediaUrls": media_urls,
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=SendMmsOptions, direction="write"
                ),
                "text": text,
                "to": to,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MmsSendResponse,
                    parse_obj_as(
                        type_=MmsSendResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def validate(
        self,
        *,
        media_urls: typing.Sequence[str],
        text: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[MmsValidationResult]:
        """
        Validate MMS message content without sending it.

        Parameters
        ----------
        media_urls : typing.Sequence[str]
            URLs of media files in this message. <br>

            See [supported media types](https://app.pinnacle.sh/supported-file-types?type=MMS).

        text : typing.Optional[str]
            Text content that accompanies the media files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[MmsValidationResult]
            Returns validation results.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "messages/validate/mms",
            method="POST",
            json={
                "mediaUrls": media_urls,
                "text": text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MmsValidationResult,
                    parse_obj_as(
                        type_=MmsValidationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ScheduledSendResponseConfig(UniversalBaseModel):
    """
    Configuration for the scheduled message.
    """

    send_at: typing_extensions.Annotated[str, FieldMetadata(alias="sendAt")] = pydantic.Field()
    """
    When the message will be sent in ISO 8601 format.
    """

    recurrence: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS cron expression used for recurring messages.
    """

    timezone: str = pydantic.Field()
    """
    IANA timezone identifier.
    """

    end_date: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="endDate")] = pydantic.Field(
        default=None
    )
    """
    When the recurring messages stops in ISO 8601 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.schedule_cancel_result import ScheduleCancelResult
from .raw_client import AsyncRawScheduleClient, RawScheduleClient


class ScheduleClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawScheduleClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawScheduleClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawScheduleClient
        """
        return self._raw_client

    def cancel(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ScheduleCancelResult:
        """
        Cancel a scheduled message or blast. <br>

        Works for both individual scheduled messages and scheduled blasts. Use the `scheduleId` returned when the message or blast was scheduled.

        Parameters
        ----------
        id : str
            Unique identifier of the scheduled message. This identifier is a string that always begins with the prefix `msg_sched_`, for example: `msg_sched_1234567890`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ScheduleCancelResult
            Successfully cancelled the scheduled message or blast.

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.messages.schedule.cancel(
            id="id",
        )
        """
        _response = self._raw_client.cancel(id, request_options=request_options)
        return _response.data


class AsyncScheduleClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawScheduleClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawScheduleClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawScheduleClient
        """
        return self._raw_client

    async def cancel(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ScheduleCancelResult:
        """
        Cancel a scheduled message or blast. <br>

        Works for both individual scheduled messages and scheduled blasts. Use the `scheduleId` returned when the message or blast was scheduled.

        Parameters
        ----------
        id : str
            Unique identifier of the scheduled message. This identifier is a string that always begins with the prefix `msg_sched_`, for example: `msg_sched_1234567890`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ScheduleCancelResult
            Successfully cancelled the scheduled message or blast.

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.messages.schedule.cancel(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cancel(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .phone_capabilities import PhoneCapabilities


class Phone(UniversalBaseModel):
    """
    Information about a phone number sender in a conversation.
    """

    capabilities: PhoneCapabilities
    created_at: typing_extensions.Annotated[str, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    ISO 8601 timestamp when the phone sender was created.
    """

    is_sandbox: typing_extensions.Annotated[bool, FieldMetadata(alias="isSandbox")] = pydantic.Field()
    """
    Whether this phone number is in sandbox/testing mode.
    """

    phone_number: typing_extensions.Annotated[str, FieldMetadata(alias="phoneNumber")] = pydantic.Field()
    """
    The sender's phone number in E.164 format.
    """

    updated_at: typing_extensions.Annotated[str, FieldMetadata(alias="updatedAt")] = pydantic.Field()
    """
    ISO 8601 timestamp when the phone sender was last updated
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DlcWithExtendedBrandAndStatusMnoTcrTier = typing.Union[
    typing.Literal["A", "B", "C", "D", "E", "F", "G", "N", "T", "S"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .rcs_button_send_location_lat_long import RcsButtonSendLocationLatLong


class RcsButtonSendLocation(UniversalBaseModel):
    """
    Button that shares a specific location with the recipient when tapped.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional name or label for the location that will be displayed in the map app (e.g., "Central Park", "Home Office").
    
    If not provided, the button title will be used as the location name.
    """

    lat_long: typing_extensions.Annotated[RcsButtonSendLocationLatLong, FieldMetadata(alias="latLong")] = (
        pydantic.Field()
    )
    """
    Geographic coordinates of the location to share.
    """

    title: str = pydantic.Field()
    """
    Display text for the button.
    """

    metadata: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional additional data to attach to this button.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.payment_required_error import PaymentRequiredError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.error import Error
from ..types.message import Message
from ..types.reaction_result import ReactionResult
from .types.react_message_options import ReactMessageOptions

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawMessagesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Message]:
        """
        Retrieve a previously sent message.

        Parameters
        ----------
        id : str
            Unique identifier of the message. This identifier is a string that always begins with the prefix `msg_`, for example: `msg_1234567890`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Message]
            Successfully retrieved the message.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"messages/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Message,
                    parse_obj_as(
                        type_=Message,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def react(
        self,
        *,
        message_id: str,
        options: typing.Optional[ReactMessageOptions] = OMIT,
        reaction: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ReactionResult]:
        """
        Add or remove an emoji reaction to a previously sent message.

        Parameters
        ----------
        message_id : str
            Unique identifier of the message. This identifier is a string that always begins with the prefix `msg_`, for example: `msg_1234567890`.

        options : typing.Optional[ReactMessageOptions]

        reaction : typing.Optional[str]
            Unicode emoji to add. <br>

            Use `null` to remove existing reaction.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ReactionResult]
            Successfully added or removed a reaction from the message.
        """
        _response = self._client_wrapper.httpx_client.request(
            "messages/react",
            method="POST",
            json={
                "messageId": message_id,
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=ReactMessageOptions, direction="write"
                ),
                "reaction": reaction,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReactionResult,
                    parse_obj_as(
                        type_=ReactionResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawMessagesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Message]:
        """
        Retrieve a previously sent message.

        Parameters
        ----------
        id : str
            Unique identifier of the message. This identifier is a string that always begins with the prefix `msg_`, for example: `msg_1234567890`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Message]
            Successfully retrieved the message.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"messages/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Message,
                    parse_obj_as(
                        type_=Message,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def react(
        self,
        *,
        message_id: str,
        options: typing.Optional[ReactMessageOptions] = OMIT,
        reaction: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ReactionResult]:
        """
        Add or remove an emoji reaction to a previously sent message.

        Parameters
        ----------
        message_id : str
            Unique identifier of the message. This identifier is a string that always begins with the prefix `msg_`, for example: `msg_1234567890`.

        options : typing.Optional[ReactMessageOptions]

        reaction : typing.Optional[str]
            Unicode emoji to add. <br>

            Use `null` to remove existing reaction.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ReactionResult]
            Successfully added or removed a reaction from the message.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "messages/react",
            method="POST",
            json={
                "messageId": message_id,
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=ReactMessageOptions, direction="write"
                ),
                "reaction": reaction,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReactionResult,
                    parse_obj_as(
                        type_=ReactionResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .send_rcs_card_options_standalone_card_image_alignment import SendRcsCardOptionsStandaloneCardImageAlignment
from .send_rcs_card_options_standalone_card_orientation import SendRcsCardOptionsStandaloneCardOrientation


class RichStandaloneCardOptions(UniversalBaseModel):
    """
    Configure standalone card layout options for enhanced visual presentation.

    > **⚠️ Important Restriction**
    >
    > This option is **only valid for single card messages** with static media. Using it with multiple cards will cause the request to fail with a validation error.
    > GIFs and videos are not supported and will be rendered as vertical cards.
    """

    orientation: typing.Optional[SendRcsCardOptionsStandaloneCardOrientation] = pydantic.Field(default=None)
    """
    The orientation of the standalone card.
    """

    image_alignment: typing.Optional[SendRcsCardOptionsStandaloneCardImageAlignment] = pydantic.Field(default=None)
    """
    The alignment of the image in the standalone card. This field is ignored if orientation is VERTICAL.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ConversationByParticipantsParams(UniversalBaseModel):
    """
    Retrieve conversation information by sender and recipient.
    """

    recipient: str = pydantic.Field()
    """
    The recipient's phone number in E.164 format
    """

    sender: str = pydantic.Field()
    """
    The sender identifier. Can be either:
    - A phone number in E.164 format
    - An RCS agent ID prefixed with 'agent_'
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ....core.serialization import FieldMetadata
from ....types.message_schedule import MessageSchedule
from .blast_rcs_request_options_standalone_card import BlastRcsRequestOptionsStandaloneCard


class BlastRcsOptions(UniversalBaseModel):
    """
    Configure how your RCS blast is sent and tracked.
    """

    transcode: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Media files and card media will be transcoded to a supported RCS format. This adds a small delay to sending the message. Ignored for rich text messages.
    """

    validate_: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="validate")] = pydantic.Field(
        default=None
    )
    """
    Validate your message for any unsupported files. <br>
    
    If failed, errors will be thrown and the blast will not be sent.
    """

    standalone_card: typing.Optional[BlastRcsRequestOptionsStandaloneCard] = pydantic.Field(default=None)
    """
    Configure standalone card layout options for enhanced visual presentation.
    
    > **⚠️ Important Restriction**
    >
    > This option is **only valid for single card messages**. Using it with multiple cards will cause the request to fail with a validation error.
    """

    schedule: typing.Optional[MessageSchedule] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VcardResource(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the contact. This identifier is a string that always begins with the prefix `cc_`, for example: `cc_1234567890`.
    """

    download_url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="downloadUrl")] = (
        pydantic.Field(default=None)
    )
    """
    Presigned download url to download vCard data. <br>
    
    This link will expire in one hour.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .campaign_validation_response_errors_item import CampaignValidationResponseErrorsItem


class CampaignValidationResult(UniversalBaseModel):
    errors: typing.List[CampaignValidationResponseErrorsItem] = pydantic.Field()
    """
    List of validation errors.
    """

    is_valid: bool = pydantic.Field()
    """
    Whether the campaign is valid or not.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VettingFeedback(UniversalBaseModel):
    description: str = pydantic.Field()
    """
    Detailed description of the feedback.
    """

    display_name: typing_extensions.Annotated[str, FieldMetadata(alias="displayName")] = pydantic.Field()
    """
    Human-readable name of the feedback item.
    """

    fields: typing.List[str] = pydantic.Field()
    """
    Fields affected by this feedback item.
    """

    id: str = pydantic.Field()
    """
    Feedback item identifier.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VCardDataFileMetadata(UniversalBaseModel):
    """
    Additional information about the download URL.
    """

    expires_at: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="expiresAt")] = pydantic.Field(
        default=None
    )
    """
    When the download URL expires in ISO 8601 format.<br>
    
    Null indicates that it is permanent.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

VCardEmailSchemaTypeItem = typing.Union[
    typing.Literal[
        "PREF",
        "INTERNET",
        "X400",
        "AOL",
        "APPLELINK",
        "ATTMAIL",
        "CIS",
        "EWORLD",
        "IBMMAIL",
        "MCIMAIL",
        "POWERSHARE",
        "PRODIGY",
        "TLX",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

PhoneFeatureEnum = typing.Union[
    typing.Literal["SMS", "MMS", "VOICE", "FAX", "EMERGENCY", "HD_VOICE", "INTERNATIONAL_SMS", "LOCAL_CALLING"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.message import Message
from ..types.reaction_result import ReactionResult
from .raw_client import AsyncRawMessagesClient, RawMessagesClient
from .types.react_message_options import ReactMessageOptions

if typing.TYPE_CHECKING:
    from .blast.client import AsyncBlastClient, BlastClient
    from .mms.client import AsyncMmsClient, MmsClient
    from .rcs.client import AsyncRcsClient, RcsClient
    from .schedule.client import AsyncScheduleClient, ScheduleClient
    from .sms.client import AsyncSmsClient, SmsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MessagesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMessagesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._sms: typing.Optional[SmsClient] = None
        self._mms: typing.Optional[MmsClient] = None
        self._rcs: typing.Optional[RcsClient] = None
        self._blast: typing.Optional[BlastClient] = None
        self._schedule: typing.Optional[ScheduleClient] = None

    @property
    def with_raw_response(self) -> RawMessagesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMessagesClient
        """
        return self._raw_client

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Message:
        """
        Retrieve a previously sent message.

        Parameters
        ----------
        id : str
            Unique identifier of the message. This identifier is a string that always begins with the prefix `msg_`, for example: `msg_1234567890`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Message
            Successfully retrieved the message.

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.messages.get(
            id="msg_1234567890",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def react(
        self,
        *,
        message_id: str,
        options: typing.Optional[ReactMessageOptions] = OMIT,
        reaction: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ReactionResult:
        """
        Add or remove an emoji reaction to a previously sent message.

        Parameters
        ----------
        message_id : str
            Unique identifier of the message. This identifier is a string that always begins with the prefix `msg_`, for example: `msg_1234567890`.

        options : typing.Optional[ReactMessageOptions]

        reaction : typing.Optional[str]
            Unicode emoji to add. <br>

            Use `null` to remove existing reaction.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReactionResult
            Successfully added or removed a reaction from the message.

        Examples
        --------
        from rcs import Pinnacle
        from rcs.messages import ReactMessageOptions

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.messages.react(
            message_id="msg_1234567890",
            options=ReactMessageOptions(
                force=True,
            ),
            reaction="👍",
        )
        """
        _response = self._raw_client.react(
            message_id=message_id, options=options, reaction=reaction, request_options=request_options
        )
        return _response.data

    @property
    def sms(self):
        if self._sms is None:
            from .sms.client import SmsClient  # noqa: E402

            self._sms = SmsClient(client_wrapper=self._client_wrapper)
        return self._sms

    @property
    def mms(self):
        if self._mms is None:
            from .mms.client import MmsClient  # noqa: E402

            self._mms = MmsClient(client_wrapper=self._client_wrapper)
        return self._mms

    @property
    def rcs(self):
        if self._rcs is None:
            from .rcs.client import RcsClient  # noqa: E402

            self._rcs = RcsClient(client_wrapper=self._client_wrapper)
        return self._rcs

    @property
    def blast(self):
        if self._blast is None:
            from .blast.client import BlastClient  # noqa: E402

            self._blast = BlastClient(client_wrapper=self._client_wrapper)
        return self._blast

    @property
    def schedule(self):
        if self._schedule is None:
            from .schedule.client import ScheduleClient  # noqa: E402

            self._schedule = ScheduleClient(client_wrapper=self._client_wrapper)
        return self._schedule


class AsyncMessagesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMessagesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._sms: typing.Optional[AsyncSmsClient] = None
        self._mms: typing.Optional[AsyncMmsClient] = None
        self._rcs: typing.Optional[AsyncRcsClient] = None
        self._blast: typing.Optional[AsyncBlastClient] = None
        self._schedule: typing.Optional[AsyncScheduleClient] = None

    @property
    def with_raw_response(self) -> AsyncRawMessagesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMessagesClient
        """
        return self._raw_client

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Message:
        """
        Retrieve a previously sent message.

        Parameters
        ----------
        id : str
            Unique identifier of the message. This identifier is a string that always begins with the prefix `msg_`, for example: `msg_1234567890`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Message
            Successfully retrieved the message.

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.messages.get(
                id="msg_1234567890",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def react(
        self,
        *,
        message_id: str,
        options: typing.Optional[ReactMessageOptions] = OMIT,
        reaction: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ReactionResult:
        """
        Add or remove an emoji reaction to a previously sent message.

        Parameters
        ----------
        message_id : str
            Unique identifier of the message. This identifier is a string that always begins with the prefix `msg_`, for example: `msg_1234567890`.

        options : typing.Optional[ReactMessageOptions]

        reaction : typing.Optional[str]
            Unicode emoji to add. <br>

            Use `null` to remove existing reaction.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ReactionResult
            Successfully added or removed a reaction from the message.

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle
        from rcs.messages import ReactMessageOptions

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.messages.react(
                message_id="msg_1234567890",
                options=ReactMessageOptions(
                    force=True,
                ),
                reaction="👍",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.react(
            message_id=message_id, options=options, reaction=reaction, request_options=request_options
        )
        return _response.data

    @property
    def sms(self):
        if self._sms is None:
            from .sms.client import AsyncSmsClient  # noqa: E402

            self._sms = AsyncSmsClient(client_wrapper=self._client_wrapper)
        return self._sms

    @property
    def mms(self):
        if self._mms is None:
            from .mms.client import AsyncMmsClient  # noqa: E402

            self._mms = AsyncMmsClient(client_wrapper=self._client_wrapper)
        return self._mms

    @property
    def rcs(self):
        if self._rcs is None:
            from .rcs.client import AsyncRcsClient  # noqa: E402

            self._rcs = AsyncRcsClient(client_wrapper=self._client_wrapper)
        return self._rcs

    @property
    def blast(self):
        if self._blast is None:
            from .blast.client import AsyncBlastClient  # noqa: E402

            self._blast = AsyncBlastClient(client_wrapper=self._client_wrapper)
        return self._blast

    @property
    def schedule(self):
        if self._schedule is None:
            from .schedule.client import AsyncScheduleClient  # noqa: E402

            self._schedule = AsyncScheduleClient(client_wrapper=self._client_wrapper)
        return self._schedule



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .validation_error_details import ValidationErrorDetails


class ValidationResults(UniversalBaseModel):
    errors: typing.List[ValidationErrorDetails] = pydantic.Field()
    """
    List of validation errors, if any.
    """

    is_valid: typing_extensions.Annotated[bool, FieldMetadata(alias="isValid")] = pydantic.Field()
    """
    Whether the brand data is valid.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .button_clicked import ButtonClicked
from .mms_content import MmsContent
from .rcs_content import RcsContent
from .sms_content import SmsContent

MessageContent = typing.Union[SmsContent, MmsContent, RcsContent, ButtonClicked]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ....core.serialization import FieldMetadata
from ....types.message_schedule import MessageSchedule


class BlastMmsOptions(UniversalBaseModel):
    """
    Additional settings to customize MMS blast delivery.
    """

    validate_: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="validate")] = pydantic.Field(
        default=None
    )
    """
    Set to true to validate the message before sending. <br>
    
    If validation fails, the blast is not sent.
    """

    schedule: typing.Optional[MessageSchedule] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DlcWithExtendedBrandAndStatusMnoBrandTier = typing.Union[
    typing.Literal["TOP", "HIGH_MID", "LOW_MID", "LOW"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .toll_free_autofill_response import TollFreeAutofillResponse
    from .toll_free_campaign_help_keywords import TollFreeCampaignHelpKeywords
    from .toll_free_campaign_keywords import TollFreeCampaignKeywords
    from .toll_free_campaign_links import TollFreeCampaignLinks
    from .toll_free_campaign_opt_in import TollFreeCampaignOptIn
    from .toll_free_campaign_opt_in_keywords import TollFreeCampaignOptInKeywords
    from .toll_free_campaign_options import TollFreeCampaignOptions
    from .toll_free_campaign_use_case import TollFreeCampaignUseCase
_dynamic_imports: typing.Dict[str, str] = {
    "TollFreeAutofillResponse": ".toll_free_autofill_response",
    "TollFreeCampaignHelpKeywords": ".toll_free_campaign_help_keywords",
    "TollFreeCampaignKeywords": ".toll_free_campaign_keywords",
    "TollFreeCampaignLinks": ".toll_free_campaign_links",
    "TollFreeCampaignOptIn": ".toll_free_campaign_opt_in",
    "TollFreeCampaignOptInKeywords": ".toll_free_campaign_opt_in_keywords",
    "TollFreeCampaignOptions": ".toll_free_campaign_options",
    "TollFreeCampaignUseCase": ".toll_free_campaign_use_case",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "TollFreeAutofillResponse",
    "TollFreeCampaignHelpKeywords",
    "TollFreeCampaignKeywords",
    "TollFreeCampaignLinks",
    "TollFreeCampaignOptIn",
    "TollFreeCampaignOptInKeywords",
    "TollFreeCampaignOptions",
    "TollFreeCampaignUseCase",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .button_clicked_data_button_raw import ButtonClickedDataButtonRaw
from .button_clicked_data_button_type import ButtonClickedDataButtonType


class ButtonClickedDataButton(UniversalBaseModel):
    """
    Information about the button that was clicked.
    """

    type: typing.Optional[ButtonClickedDataButtonType] = pydantic.Field(default=None)
    """
    Type of button clicked. 
    - `CARD` for card buttons, 
    - `QUICK_REPLY` for quick replies. 
    - `null` if we cannot determine the button type. This is a rare edge case and is mainly here for backward compatibility.
    """

    raw: ButtonClickedDataButtonRaw = pydantic.Field()
    """
    Raw button data. Contains the entire button that was clicked by the user. In rare cases where we cannot determine the exact button, this will return only the button title.
    """

    payload: typing.Optional[str] = pydantic.Field(default=None)
    """
    Extracted payload from the button's `payload` field, if provided. Provides quick access to any payload that was attached to the button.
    """

    metadata: typing.Optional[str] = pydantic.Field(default=None)
    """
    Additional metadata attached to the button's `metadata` field, if provided.
    """

    clicks: int = pydantic.Field()
    """
    Number of times the button has been clicked.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ....core.serialization import FieldMetadata
from .download_options import DownloadOptions


class UploadFileOptions(UniversalBaseModel):
    """
    Additional configurations for your file.
    """

    delete_at: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="deleteAt")] = pydantic.Field(
        default=None
    )
    """
    Set a deletion date for your file in ISO 8601 format. After this date, the file will be automatically deleted from our storage.<br>
    
    If this field is not provided, the file will not be deleted. You can still schedule deletion or delete the file manually in the Storage page in the dashboard.
    """

    download: typing.Optional[DownloadOptions] = pydantic.Field(default=None)
    """
    Configure download settings for your uploaded file.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class RcsCapabilityActions(UniversalBaseModel):
    """
    Supported button action types inside the rich cards or quick replies. Each key is a button action type that defines a specific button type in the Pinnacle API.
    """

    open_url: typing_extensions.Annotated[bool, FieldMetadata(alias="openUrl")] = pydantic.Field()
    """
    Buttons that open URLs are supported
    """

    call: bool = pydantic.Field()
    """
    Buttons that make phone calls are supported
    """

    trigger: bool = pydantic.Field()
    """
    Buttons that trigger actions are supported
    """

    request_user_location: typing_extensions.Annotated[bool, FieldMetadata(alias="requestUserLocation")] = (
        pydantic.Field()
    )
    """
    Buttons that request user location are supported
    """

    schedule_event: typing_extensions.Annotated[bool, FieldMetadata(alias="scheduleEvent")] = pydantic.Field()
    """
    Buttons are supported
    """

    send_location: typing_extensions.Annotated[bool, FieldMetadata(alias="sendLocation")] = pydantic.Field()
    """
    Buttons that send location are supported
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .enhanced_contact_info import EnhancedContactInfo


class RetrievePhoneNumberDetailsOptions(UniversalBaseModel):
    """
    Customize your lookup with additional options.
    """

    force: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Allows you to force a fresh lookup from primary sources instead of cached data. <br>
    
    Fresh lookups will take longer to complete than cached lookups.
    """

    risk: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Include a fraud risk and security analysis.
    """

    enhanced_contact_info: typing.Optional[EnhancedContactInfo] = pydantic.Field(default=None)
    """
    Additional information to tailor lookup.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .webhook_event_enum import WebhookEventEnum


class CreateAndAttachWebhookByUrlParams(UniversalBaseModel):
    """
    Create a new webhook and immediately attach it to your phone number.
    """

    name: str = pydantic.Field()
    """
    Name of your new webhook.
    """

    url: str = pydantic.Field()
    """
    HTTPS endpoint where your webhook events will be sent as HTTP POST requests. <br>
    
    Ensure this URL is publicly accessible and ready to handle incoming webhook calls.
    """

    event: typing.Optional[WebhookEventEnum] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
